"""SDK to communicate with Nazca4.0 system
    To get info about system:
            sdk.modules returns list of all modules
            sdk.variables returns dictionary with key:module value: available variables
    Example:
            sdk.modules -> [Module1, Module2, ... ]
            sdk.variables ->{
                                Module_Name: [Var1, Var2, ... ]
                            }
"""

from datetime import datetime
from datetime import timedelta
from pydantic import ValidationError
from nazca4sdk.cache import Cache
from nazca4sdk.datahandling.data_mod import Data
from nazca4sdk.datahandling.variable_verificator import VariableIntervalInfo, \
    VariableIntervalSubtractionInfo


class SDK:
    """SDK by Nazca4 system"""

    def __init__(self):
        """ Initializing the system, checking connection and caching system configuration"""

        self._system_cache = Cache()
        if not self._system_cache.load:
            print("Init SDK failed")
            self.modules = []
            self.variables = []
            self.types = []
        else:
            self.modules = self._system_cache.modules
            self.variables = self._system_cache.variables

    def variable_over_day(self, module_name, variable_names, start_date, end_date):
        """Get variables in specific time range

        Args:
            module_name - name of module,
            variable_names - list of variable names,
            start_time - beginning of the time range
            stop_time - ending of the time range

        Returns:
            DataFrame: Variable values from selected time range
        """
        try:
            data = {'module_name': module_name,
                    'variable_names': variable_names,
                    'start_date': start_date,
                    'end_date': end_date}
            variable_info = VariableIntervalInfo(**data)
            result = self._system_cache.variable_over_day(variable_info.module_name,
                                                          variable_info.variable_names,
                                                          variable_info.start_date,
                                                          variable_info.end_date)
            if result is None:
                return None
            return result.to_df()
        except ValidationError as error:
            print(error.json())
            return None

    def variable_over_time(self, module_name: str,
                           variable_names: list,
                           time_amount: int,
                           time_unit: str):
        """ Get variables in specific time range

        Args:
            module_name - name of module,
            variable_names - list of variable names,
            time_amount - beginning of the time range
            time_unit - 'DAY','HOUR'...

        Returns:
            DataFrame: Variable values from selected time range
        """
        try:
            data = {'module_name': module_name,
                    'variable_names': variable_names,
                    'time_amount': time_amount,
                    'time_unit': time_unit}
            variable_info = VariableIntervalSubtractionInfo(**data)

            end_date = datetime.utcnow()
            start_date = end_date - self.__get_time_delta(time_unit, time_amount)

            result = self._system_cache.variable_over_day(variable_info.module_name,
                                                          variable_info.variable_names,
                                                          start_date,
                                                          end_date)
            if result is None:
                return None
            return result.to_df()
        except ValidationError as error:
            print(error.json())
            return None

    @staticmethod
    def __get_time_delta(time_unit, time_amount):
        if time_unit == 'SECOND':
            return timedelta(seconds=time_amount)
        if time_unit == 'MINUTE':
            return timedelta(minutes=time_amount)
        if time_unit == 'HOUR':
            return timedelta(hours=time_amount)
        if time_unit == 'DAY':
            return timedelta(days=time_amount)
        if time_unit == 'WEEK':
            return timedelta(weeks=time_amount)
        if time_unit == 'YEAR':
            return timedelta(days=time_amount * 365)
        return None

    @staticmethod
    def to_list(data):
        """Function to modify data to list structure

        Args:
            data - input data in DataFrame format

        Returns:
            List: input data as a list
        """
        data_frame = Data(data)
        try:
            return data_frame.data_to_list()
        except RuntimeError as runtime_error:
            print(f'Problem with conversion to list - {runtime_error}')
            return None

    @staticmethod
    def to_numpy(data):
        """Function to modify data to numpy array structure

        Args:
            data - input data in DataFrame format

        Returns:
            np.array: input data as a numpy array
        """
        data_frame = Data(data)
        try:
            return data_frame.data_to_numpy()
        except RuntimeError as runtime_error:
            print(f'Problem with conversion to numpy array - {runtime_error}')
            return None